/*
    Project: TCGINA (Setup V1.0m)
    Version: V1.15
    File   : fileinfo.c

    Copyright (c) 2005-2006 Author of TCGINA DLL. All rights reserved.

    Covered by TrueCrypt License 2.1 the full text of which is contained
    in the file License.txt included in TCGINA binary and source code
    distribution archives.

    The copyright notice of this source code file and the following public key
    must be retained in full.

-----BEGIN PGP PUBLIC KEY BLOCK-----
Version: GnuPG v1.2.4 (MingW32)

mQGiBEI3cVwRBADCRqJ1+AKeMw4BagfF9yNdobBBJXTFQ0R3RJHRAOoKMrT5rxMn
zjRvN9SugcTAftMRVTmsHVZcve6jO2yLHJLg2QgtHiRKdbLV9MTrzYtxXUL1Ez0A
3Mq6ww/h7EdwG5VlPX8uw++Y2aX2UHvMgRbImcLDz+/8qT1FR6dvujjyhwCgmUR9
/4+H8QkCw7M2ekFsODzSBzsEAIwTWuqYfwPRISyGTL/ajZC2WVxLNCC1NmvxzRMe
v/Mg344nxOrMG6MNTNYJdq/3dkiWBzXcpy5zWAVMxAiMDhKSytdk5mdvPI69wZtS
UZ92qXxvm/6fPjIoFy1hSII0LExLuS5p0MMZ7yFFEjZ4CQG/fPE04aUF0zAVgQQ8
OOjJA/0U+8ZlJnPF5cMfvODlOzjfG+LFhWS/t4NhJ12CWeOyn8+enAiXQsKyKyll
JtE9zf8aQKhfS1KJ/ZLZphmnN5LqQLOi/HVsAIK4/KPRrv7M4Z+atChLbmvvR31I
AcTOoDqxj4d8D8yiV8i+ZfLAXREPVLXc3c/W5gBrYwyp5bmpG7QUQXV0aG9yIG9m
IFRDR0lOQSBETEyIXgQTEQIAHgUCQjdxXAIbAwYLCQgHAwIDFQIDAxYCAQIeAQIX
gAAKCRD/R4xyQJdnzk9iAJ9rTCyGTUioEOJEU2w779rVzXoUkQCeO8zdw6QakpiA
uc2XmzGb6JscCEA=
=2DjS
-----END PGP PUBLIC KEY BLOCK-----
*/

#include <windows.h>
#include <tchar.h>
#include <tomcrypt.h>
#include <tc.h>
#include "fileinfo.h"

typedef struct {
    WORD wLanguage;
    WORD wCodePage;
} LANG_AND_CODEPAGE, *PLANG_AND_CODEPAGE;

LPCTSTR GetIniFileName(void)
{
    static TCHAR szIniFileName[MAX_PATH] = TEXT("");

    if (*szIniFileName == TEXT('\0')) {
        if (GetModuleFileName(NULL, szIniFileName, MAX_PATH)) {
            LPTSTR pszTemp = _tcsrchr(szIniFileName, TEXT('.'));
            if (pszTemp != NULL) {
                pszTemp[1] = TEXT('\0');
                TC_AppendString(pszTemp, MAX_PATH, TEXT("INI"));
            } else {
                *szIniFileName = TEXT('\0');
            }
        } else {
            *szIniFileName = TEXT('\0');
        }
    }

    return szIniFileName;
}

BOOL GetFileInfo(PFILE_INFO pFileInfo, LPTSTR pszFileName)
{
    HANDLE       hFile;
    DWORD        dwFileVersionInfoSize;
    DWORD        dwResult;

    memset(pFileInfo, '\0', sizeof(*pFileInfo));
    hFile = FindFirstFile(pszFileName, &pFileInfo->fd);
    if (hFile == INVALID_HANDLE_VALUE) {
        TC_CopyString(pFileInfo->fd.cFileName,
          sizeof(pFileInfo->fd.cFileName)/sizeof(TCHAR), pszFileName);
        return FALSE;
    }
    FindClose(hFile);
    pFileInfo->fExists = TRUE;
    TC_CopyString(pFileInfo->fd.cFileName,
      sizeof(pFileInfo->fd.cFileName)/sizeof(TCHAR), pszFileName);

    if (pFileInfo->fd.nFileSizeHigh == 0 && pFileInfo->fd.nFileSizeLow != 0 &&
      pFileInfo->fd.nFileSizeLow < 0x100000) {
        void *pData = malloc(pFileInfo->fd.nFileSizeLow);
        if (pData) {
            hFile = CreateFile(pszFileName, GENERIC_READ, FILE_SHARE_READ, NULL,
            OPEN_EXISTING, 0, NULL);
            if (hFile != INVALID_HANDLE_VALUE) {
                DWORD dwFileSize;
                if (ReadFile(hFile, pData, pFileInfo->fd.nFileSizeLow, &dwFileSize, NULL) &&
                  pFileInfo->fd.nFileSizeLow == dwFileSize) {
                    hash_state md;
                    TCHAR      szHash[2*FILE_INFO_HASH_SIZE+1];
                    unsigned   i;
                    sha1_init(&md);
                    sha1_process(&md, pData, dwFileSize);
                    sha1_done(&md, pFileInfo->rgbHashValue);
                    for (i = 0; i < FILE_INFO_HASH_SIZE; i++) {
                        wsprintf(szHash+i*2, TEXT("%02X"),
                          (unsigned)pFileInfo->rgbHashValue[i]);
                    }
                    szHash[i*2] = TEXT('\0');
                    GetPrivateProfileString(TEXT("SHA1"), szHash, TEXT(""),
                      pFileInfo->szInfo, MAX_PATH, GetIniFileName());
                }
                CloseHandle(hFile);
            }
            free(pData);
        }
    }

    hFile = CreateFile(pszFileName, GENERIC_WRITE, 0, NULL, OPEN_EXISTING, 0,
      NULL);
    if (hFile != INVALID_HANDLE_VALUE) {
        CloseHandle(hFile);
    } else {
        pFileInfo->fInUse = TRUE;
    }

    dwFileVersionInfoSize = GetFileVersionInfoSize(pszFileName, &dwResult);
    if (dwFileVersionInfoSize) {
        PVOID pBlock = malloc(dwFileVersionInfoSize);
        if (pBlock) {
            if (GetFileVersionInfo(pszFileName, 0, dwFileVersionInfoSize,
              pBlock)) {
                VS_FIXEDFILEINFO   *ffi;
                UINT               uLen;
                PLANG_AND_CODEPAGE pTranslation;
                TCHAR              szValueName[64];
                LPTSTR             pszValueData;
                if (VerQueryValue(pBlock, TEXT("\\"), (LPVOID *)&ffi, &uLen) &&
                  ffi->dwSignature == 0xfeef04bd) {
                    ((DWORD*)&pFileInfo->ullVersion)[0] = ffi->dwFileVersionLS;
                    ((DWORD*)&pFileInfo->ullVersion)[1] = ffi->dwFileVersionMS;
                    pFileInfo->uMajorVersion = ffi->dwFileVersionMS >> 16;
                    pFileInfo->uMinorVersion = ffi->dwFileVersionMS & 0xffff;
                    pFileInfo->uRelease      = ffi->dwFileVersionLS >> 16;
                    pFileInfo->uBuild        = ffi->dwFileVersionLS & 0xffff;
#if 0
                    if (pFileInfo->uBuild == 0 && pFileInfo->uRelease <= 26) {
                        if (pFileInfo->uRelease == 0) {
                            _sntprintf(pFileInfo->szVersion,
                              sizeof(pFileInfo->szVersion)/sizeof(TCHAR),
                              TEXT("%u.%u"),
                              pFileInfo->uMajorVersion,
                              pFileInfo->uMinorVersion);
                        } else {
                            _sntprintf(pFileInfo->szVersion,
                              sizeof(pFileInfo->szVersion)/sizeof(TCHAR),
                              TEXT("%u.%u%c"),
                              pFileInfo->uMajorVersion,
                              pFileInfo->uMinorVersion,
                              (TCHAR)(TEXT('a')+(pFileInfo->uRelease-1)));
                        }
                    } else {
#endif
                        _sntprintf(pFileInfo->szVersion,
                          sizeof(pFileInfo->szVersion)/sizeof(TCHAR),
                          TEXT("%u.%u.%u.%u"),
                          pFileInfo->uMajorVersion,
                          pFileInfo->uMinorVersion,
                          pFileInfo->uRelease,
                          pFileInfo->uBuild);
#if 0
                    }
#endif
                }

                uLen = 0;
                VerQueryValue(pBlock, TEXT("\\VarFileInfo\\Translation"),
                  (LPVOID*)&pTranslation, &uLen);

                if (uLen >= sizeof(LANG_AND_CODEPAGE)) {
                    _sntprintf(szValueName, sizeof(szValueName)/sizeof(TCHAR),
                      TEXT("\\StringFileInfo\\%04x%04x\\FileVersion"),
                      (unsigned)pTranslation[0].wLanguage,
                      (unsigned)pTranslation[0].wCodePage);
                    if (VerQueryValue(pBlock, szValueName,
                      (LPVOID*)&pszValueData, &uLen)) {
                        TC_CopyString(pFileInfo->szFileVersion,
                          sizeof(pFileInfo->szFileVersion)/sizeof(TCHAR),
                          pszValueData);
                    }
                    _sntprintf(szValueName, sizeof(szValueName)/sizeof(TCHAR),
                      TEXT("\\StringFileInfo\\%04x%04x\\ProductName"),
                      (unsigned)pTranslation[0].wLanguage,
                      (unsigned)pTranslation[0].wCodePage);
                    if (VerQueryValue(pBlock, szValueName,
                      (LPVOID*)&pszValueData, &uLen)) {
                        TC_CopyString(pFileInfo->szProductName,
                          sizeof(pFileInfo->szProductName)/sizeof(TCHAR),
                          pszValueData);
                    }
                    _sntprintf(szValueName, sizeof(szValueName)/sizeof(TCHAR),
                      TEXT("\\StringFileInfo\\%04x%04x\\ProductVersion"),
                      (unsigned)pTranslation[0].wLanguage,
                      (unsigned)pTranslation[0].wCodePage);
                    if (VerQueryValue(pBlock, szValueName,
                      (LPVOID*)&pszValueData, &uLen)) {
                        TC_CopyString(pFileInfo->szProductVersion,
                          sizeof(pFileInfo->szProductVersion)/sizeof(TCHAR),
                          pszValueData);
                    }
                    _sntprintf(szValueName, sizeof(szValueName)/sizeof(TCHAR),
                      TEXT("\\StringFileInfo\\%04x%04x\\LegalCopyright"),
                      (unsigned)pTranslation[0].wLanguage,
                      (unsigned)pTranslation[0].wCodePage);
                    if (VerQueryValue(pBlock, szValueName,
                      (LPVOID*)&pszValueData, &uLen)) {
                        TC_CopyString(pFileInfo->szCopyright,
                          sizeof(pFileInfo->szCopyright)/sizeof(TCHAR),
                          pszValueData);
                    }
                    _sntprintf(szValueName, sizeof(szValueName)/sizeof(TCHAR),
                      TEXT("\\StringFileInfo\\%04x%04x\\OriginalFilename"),
                      (unsigned)pTranslation[0].wLanguage,
                      (unsigned)pTranslation[0].wCodePage);
                    if (VerQueryValue(pBlock, szValueName,
                      (LPVOID*)&pszValueData, &uLen)) {
                        TC_CopyString(pFileInfo->szOriginalFileName,
                          sizeof(pFileInfo->szOriginalFileName)/sizeof(TCHAR),
                          pszValueData);
                        _tcsupr(pFileInfo->szOriginalFileName);
                    }
                    _sntprintf(szValueName, sizeof(szValueName)/sizeof(TCHAR),
                      TEXT("\\StringFileInfo\\%04x%04x\\CompanyName"),
                      (unsigned)pTranslation[0].wLanguage,
                      (unsigned)pTranslation[0].wCodePage);
                    if (VerQueryValue(pBlock, szValueName,
                      (LPVOID*)&pszValueData, &uLen)) {
                        TC_CopyString(pFileInfo->szCompanyName,
                          sizeof(pFileInfo->szCompanyName)/sizeof(TCHAR),
                          pszValueData);
                    }
                    _sntprintf(szValueName, sizeof(szValueName)/sizeof(TCHAR),
                      TEXT("\\StringFileInfo\\%04x%04x\\FileDescription"),
                      (unsigned)pTranslation[0].wLanguage,
                      (unsigned)pTranslation[0].wCodePage);
                    if (VerQueryValue(pBlock, szValueName,
                      (LPVOID*)&pszValueData, &uLen)) {
                        TC_CopyString(pFileInfo->szFileDescription,
                          sizeof(pFileInfo->szFileDescription)/sizeof(TCHAR),
                          pszValueData);
                    }
                }
            }
            free(pBlock);
        }
    }

    return TRUE;
}

BOOL GetInfoItem(LPCTSTR pszInfo, DWORD dwIndex, LPTSTR pszDest, DWORD nSize)
{
    LPCTSTR pszEnd;
    DWORD   dwItemLen;

    *pszDest = TEXT('\0');
    while (dwIndex--) {
        pszInfo = _tcschr(pszInfo, TEXT(';'));
        if (pszInfo == NULL) return FALSE;
        pszInfo++;
    }
    pszEnd = _tcschr(pszInfo, TEXT(';'));
    if (pszEnd == NULL) {
        pszEnd = pszInfo + _tcslen(pszInfo);
    }
    dwItemLen = (DWORD)(pszEnd - pszInfo);

    if (nSize <= dwItemLen) {
        return FALSE;
    }
    memmove(pszDest, pszInfo, dwItemLen*sizeof(TCHAR));
    pszDest[dwItemLen] = TEXT('\0');

    return TRUE;
}