/*
    Project: TCGINA (Setup V1.0m)
    Version: V1.15
    File   : reg.c

    Copyright (c) 2005-2006 Author of TCGINA DLL. All rights reserved.

    Covered by TrueCrypt License 2.1 the full text of which is contained
    in the file License.txt included in TCGINA binary and source code
    distribution archives.

    The copyright notice of this source code file and the following public key
    must be retained in full.

-----BEGIN PGP PUBLIC KEY BLOCK-----
Version: GnuPG v1.2.4 (MingW32)

mQGiBEI3cVwRBADCRqJ1+AKeMw4BagfF9yNdobBBJXTFQ0R3RJHRAOoKMrT5rxMn
zjRvN9SugcTAftMRVTmsHVZcve6jO2yLHJLg2QgtHiRKdbLV9MTrzYtxXUL1Ez0A
3Mq6ww/h7EdwG5VlPX8uw++Y2aX2UHvMgRbImcLDz+/8qT1FR6dvujjyhwCgmUR9
/4+H8QkCw7M2ekFsODzSBzsEAIwTWuqYfwPRISyGTL/ajZC2WVxLNCC1NmvxzRMe
v/Mg344nxOrMG6MNTNYJdq/3dkiWBzXcpy5zWAVMxAiMDhKSytdk5mdvPI69wZtS
UZ92qXxvm/6fPjIoFy1hSII0LExLuS5p0MMZ7yFFEjZ4CQG/fPE04aUF0zAVgQQ8
OOjJA/0U+8ZlJnPF5cMfvODlOzjfG+LFhWS/t4NhJ12CWeOyn8+enAiXQsKyKyll
JtE9zf8aQKhfS1KJ/ZLZphmnN5LqQLOi/HVsAIK4/KPRrv7M4Z+atChLbmvvR31I
AcTOoDqxj4d8D8yiV8i+ZfLAXREPVLXc3c/W5gBrYwyp5bmpG7QUQXV0aG9yIG9m
IFRDR0lOQSBETEyIXgQTEQIAHgUCQjdxXAIbAwYLCQgHAwIDFQIDAxYCAQIeAQIX
gAAKCRD/R4xyQJdnzk9iAJ9rTCyGTUioEOJEU2w779rVzXoUkQCeO8zdw6QakpiA
uc2XmzGb6JscCEA=
=2DjS
-----END PGP PUBLIC KEY BLOCK-----
*/

#include <windows.h>
#include <tchar.h>

BOOL RegKeyExists(HKEY hRootKey, LPCTSTR pszKeyName)
{
    HKEY hKey;

    if (RegOpenKeyEx(hRootKey, pszKeyName, 0, KEY_READ, &hKey) ==
      ERROR_SUCCESS) {
        RegCloseKey(hKey);
        return TRUE;
    }

    return FALSE;
}

BOOL RegKeyIsEmpty(HKEY hRootKey, LPCTSTR pszKeyName)
{
    HKEY  hKey;
    BOOL  fResult;
    DWORD cSubKeys;
    DWORD cValues;

    if (RegOpenKeyEx(hRootKey, pszKeyName, 0, KEY_READ, &hKey) !=
      ERROR_SUCCESS) {
        return FALSE;
    }

    fResult = FALSE;
    if (RegQueryInfoKey(hKey, NULL, NULL, NULL, &cSubKeys,
      NULL, NULL, &cValues, NULL, NULL, NULL, NULL) == ERROR_SUCCESS) {
        if (cSubKeys == 0 && cValues == 0) {
            fResult = TRUE;
        }
    }
    RegCloseKey(hKey);

    return fResult;
}

BOOL RegValueExists(HKEY hRootKey, LPCTSTR pszKeyName, LPCTSTR pszValueName)
{
    HKEY hKey;
    BOOL fResult = FALSE;

    if (RegOpenKeyEx(hRootKey, pszKeyName, 0, KEY_READ, &hKey) ==
      ERROR_SUCCESS) {
        DWORD cbSize;
        DWORD dwType;
        LONG  lResult;

        cbSize = 0;
        lResult = RegQueryValueEx(hKey, pszValueName, NULL, &dwType,
          NULL, &cbSize);
        if (lResult == ERROR_SUCCESS || lResult == ERROR_MORE_DATA) {
            fResult = TRUE;
        }

        RegCloseKey(hKey);
    }

    return fResult;
}

BOOL RegRemoveKey(HKEY hRootKey, LPCTSTR pszKeyName)
{
    HKEY     hKey;
    LONG     lResult;
    TCHAR    szSubKeyName[MAX_PATH];
    DWORD    dwSize;
    FILETIME ftLastWriteTime;

    if (RegDeleteKey(hRootKey, pszKeyName) == ERROR_SUCCESS) {
        return TRUE;
    }

    lResult = RegOpenKeyEx(hRootKey, pszKeyName, 0, KEY_READ, &hKey);
    if (lResult != ERROR_SUCCESS) {
        return (lResult == ERROR_FILE_NOT_FOUND) ? TRUE : FALSE;
    }

    dwSize = sizeof(szSubKeyName)/sizeof(TCHAR);
    while (RegEnumKeyEx(hKey, 0, szSubKeyName, &dwSize, NULL,
      NULL, NULL, &ftLastWriteTime) == ERROR_SUCCESS) {
        if (!RegRemoveKey(hKey, szSubKeyName)) {
            RegCloseKey(hKey);
            return FALSE;
        }
        dwSize = sizeof(szSubKeyName)/sizeof(TCHAR);
    }
    RegCloseKey(hKey);

    return RegDeleteKey(hRootKey, pszKeyName) == ERROR_SUCCESS;
}

BOOL RegRemoveValue(HKEY hRootKey, LPCTSTR pszKeyName,
  LPCTSTR pszValueName)
{
    HKEY hKey;
    BOOL fResult = FALSE;

    if (RegOpenKeyEx(hRootKey, pszKeyName, 0, KEY_WRITE, &hKey) ==
      ERROR_SUCCESS) {
        if (RegDeleteValue(hKey, pszValueName) == ERROR_SUCCESS) {
            fResult = TRUE;
        }
        RegCloseKey(hKey);
    }

    return fResult;
}

BOOL RegCopyValue(HKEY hSrcRootKey, LPCTSTR pszSrcKeyName,
  LPCTSTR pszSrcValueName, HKEY hDestRootKey, LPCTSTR pszDestKeyName,
  LPCTSTR pszDestValueName)
{
    HKEY hKey;
    void *pData;
    DWORD cbSize;
    DWORD dwType;
    LONG  lResult;

    if (RegOpenKeyEx(hSrcRootKey, pszSrcKeyName, 0, KEY_READ, &hKey)
      != ERROR_SUCCESS) {
        return FALSE;
    }

    cbSize = 0;
    lResult = RegQueryValueEx(hKey, pszSrcValueName, NULL, &dwType,
      NULL, &cbSize);
    if (lResult != ERROR_SUCCESS && lResult != ERROR_MORE_DATA) {
        RegCloseKey(hKey);
        return FALSE;
    }
    pData = malloc(cbSize+1);
    if (pData == NULL) {
        RegCloseKey(hKey);
        return FALSE;
    }
    if (RegQueryValueEx(hKey, pszSrcValueName, NULL, &dwType,
      pData, &cbSize) != ERROR_SUCCESS) {
        free(pData);
        RegCloseKey(hKey);
        return FALSE;
    }
    RegCloseKey(hKey);

    if (RegCreateKeyEx(hDestRootKey, pszDestKeyName, 0, NULL, 0,
      KEY_WRITE, NULL, &hKey, NULL) != ERROR_SUCCESS) {
        free(pData);
        return FALSE;
    }
    if (RegSetValueEx(hKey, pszDestValueName, 0, dwType, pData,
      cbSize) != ERROR_SUCCESS) {
        free(pData);
        RegCloseKey(hKey);
        return FALSE;
    }

    free(pData);
    RegCloseKey(hKey);

    return TRUE;
}

BOOL RegCloneValue(HKEY hSrcRootKey, LPCTSTR pszSrcKeyName,
  LPCTSTR pszSrcValueName, HKEY hDestRootKey, LPCTSTR pszDestKeyName,
  LPCTSTR pszDestValueName) {

    if (!RegValueExists(hSrcRootKey, pszSrcKeyName, pszSrcValueName)) {
        if (!RegValueExists(hDestRootKey, pszDestKeyName, pszDestValueName)) {
            return TRUE;
        }
        return RegRemoveValue(hDestRootKey, pszDestKeyName, pszDestValueName);
    }

    return RegCopyValue(hSrcRootKey, pszSrcKeyName, pszSrcValueName,
      hDestRootKey, pszDestKeyName, pszDestValueName);
}

BOOL RegGetValueData(HKEY hRootKey, LPCTSTR pszKeyName,
  LPCTSTR pszValueName, PDWORD pdwType, PBYTE pDest, PDWORD pcbSize)
{
    HKEY  hKey;
    BOOL  fResult;

    fResult = FALSE;
    if (RegOpenKeyEx(hRootKey, pszKeyName, 0, KEY_READ, &hKey) ==
      ERROR_SUCCESS) {
        if (RegQueryValueEx(hKey, pszValueName, NULL, pdwType,
          pDest, pcbSize) == ERROR_SUCCESS) {
            fResult = TRUE;
        }
        RegCloseKey(hKey);
    }

    return fResult;
}

BOOL RegSetString(HKEY hRootKey, LPCTSTR pszKeyName, LPCTSTR pszValueName,
  DWORD dwType, LPCTSTR pszValueData)
{
    HKEY hKey;
    BOOL  fResult = FALSE;

    if (RegCreateKeyEx(hRootKey, pszKeyName, 0, NULL, 0,
      KEY_WRITE, NULL, &hKey, NULL) != ERROR_SUCCESS) {
        return FALSE;
    }
    if (RegSetValueEx(hKey, pszValueName, 0, dwType, (const BYTE*)pszValueData,
      (DWORD)(_tcslen(pszValueData)+1)*sizeof(TCHAR)) == ERROR_SUCCESS) {
        fResult = TRUE;
    }
    RegCloseKey(hKey);

    return fResult;
}

BOOL RegSetInteger(HKEY hRootKey, LPCTSTR pszKeyName, LPCTSTR pszValueName,
  DWORD dwData)
{
    HKEY hKey;
    BOOL fResult = FALSE;

    if (RegCreateKeyEx(hRootKey, pszKeyName, 0, NULL, 0,
      KEY_WRITE, NULL, &hKey, NULL) != ERROR_SUCCESS) {
        return FALSE;
    }
    if (RegSetValueEx(hKey, pszValueName, 0, REG_DWORD, (const BYTE*)&dwData,
      sizeof(DWORD)) == ERROR_SUCCESS) {
        fResult = TRUE;
    }
    RegCloseKey(hKey);

    return fResult;
}

BOOL PendingFileRenameOperations(void)
{
    HKEY  hKey;
    DWORD dwType;
    DWORD cbSize;

    if (RegOpenKeyEx(HKEY_LOCAL_MACHINE,
      TEXT("SYSTEM\\CurrentControlSet\\Control\\Session Manager"),
      0, KEY_READ, &hKey) != ERROR_SUCCESS) {
        return FALSE;
    }
    cbSize = 0;
    RegQueryValueEx(hKey, TEXT("PendingFileRenameOperations"), NULL, &dwType,
      NULL, &cbSize);
    RegCloseKey(hKey);

    return cbSize != 0;
}
