/*
    Project: TCGINA
    Version: V1.15
    File   : tckeyfile.c

    Copyright (c) 2005-2006 Author of TCGINA DLL. All rights reserved.

    Covered by TrueCrypt License 2.1 the full text of which is contained
    in the file License.txt included in TCGINA binary and source code
    distribution archives.

    The copyright notice of this source code file and the following public key
    must be retained in full.

-----BEGIN PGP PUBLIC KEY BLOCK-----
Version: GnuPG v1.2.4 (MingW32)

mQGiBEI3cVwRBADCRqJ1+AKeMw4BagfF9yNdobBBJXTFQ0R3RJHRAOoKMrT5rxMn
zjRvN9SugcTAftMRVTmsHVZcve6jO2yLHJLg2QgtHiRKdbLV9MTrzYtxXUL1Ez0A
3Mq6ww/h7EdwG5VlPX8uw++Y2aX2UHvMgRbImcLDz+/8qT1FR6dvujjyhwCgmUR9
/4+H8QkCw7M2ekFsODzSBzsEAIwTWuqYfwPRISyGTL/ajZC2WVxLNCC1NmvxzRMe
v/Mg344nxOrMG6MNTNYJdq/3dkiWBzXcpy5zWAVMxAiMDhKSytdk5mdvPI69wZtS
UZ92qXxvm/6fPjIoFy1hSII0LExLuS5p0MMZ7yFFEjZ4CQG/fPE04aUF0zAVgQQ8
OOjJA/0U+8ZlJnPF5cMfvODlOzjfG+LFhWS/t4NhJ12CWeOyn8+enAiXQsKyKyll
JtE9zf8aQKhfS1KJ/ZLZphmnN5LqQLOi/HVsAIK4/KPRrv7M4Z+atChLbmvvR31I
AcTOoDqxj4d8D8yiV8i+ZfLAXREPVLXc3c/W5gBrYwyp5bmpG7QUQXV0aG9yIG9m
IFRDR0lOQSBETEyIXgQTEQIAHgUCQjdxXAIbAwYLCQgHAwIDFQIDAxYCAQIeAQIX
gAAKCRD/R4xyQJdnzk9iAJ9rTCyGTUioEOJEU2w779rVzXoUkQCeO8zdw6QakpiA
uc2XmzGb6JscCEA=
=2DjS
-----END PGP PUBLIC KEY BLOCK-----
*/

/*
 * The method to derive the password from the keyfiles is taken from the
 * source code of TrueCrypt 4.0
 */

#include "tcpriv.h"
#include <tchar.h>

#pragma pack(push, 1)
typedef struct
{
    union {
        DWORD dwRemainder;
        BYTE  bRemainder[4];
    } data;
    DWORD dwZero;
} CRC32_REMAINDER;
#pragma pack(pop)

extern const DWORD rgdwCRC32Table[256];
#define InitCRC(cr)   cr.dwZero = 0; cr.data.dwRemainder = 0xffffffff
#define AddCRC(cr, b) cr.data.dwRemainder = \
  rgdwCRC32Table[cr.data.bRemainder[0] ^ (b)] ^ \
  *(DWORD *)&cr.data.bRemainder[1]
#define GetCRC(cr)    cr.data.dwRemainder

static BOOL TC_ReadKeyFile(LPSTR paszPassword, HANDLE hFile)
{
    CRC32_REMAINDER cr;
    BYTE            rgBuffer[0x10000];
    DWORD           i;
    DWORD           cRead;
    int             idx = 0;
    size_t          uTotalRead = 0;

    TC_Assert(paszPassword);
    TC_Assert(hFile != INVALID_HANDLE_VALUE);

    InitCRC(cr);

    while (ReadFile(hFile, rgBuffer, sizeof(rgBuffer), &cRead, NULL)) {
        if (cRead == 0) break;

        for (i = 0; i < cRead; i++) {
            AddCRC(cr, rgBuffer[i]);

            paszPassword[idx++] =
              (BYTE)(paszPassword[idx] + cr.data.bRemainder[3]);
            paszPassword[idx++] =
              (BYTE)(paszPassword[idx] + cr.data.bRemainder[2]);
            paszPassword[idx++] =
              (BYTE)(paszPassword[idx] + cr.data.bRemainder[1]);
            paszPassword[idx++] =
              (BYTE)(paszPassword[idx] + cr.data.bRemainder[0]);
            if (idx >= TC_MAX_PASSWORD_LEN) {
                idx = 0;
            }
            if (++uTotalRead >= 0x100000) return TRUE;
        }
    }

    return TRUE;
}

static BOOL TC_ReadKeyFileByName(LPSTR paszPassword, LPCTSTR pszKeyFile,
  BOOL fPreserveTimeStamp)
{
    HANDLE   hFile;
    HANDLE   hFile2;
    FILETIME ftCreation;
    FILETIME ftLastAccess;
    FILETIME ftLastWrite;
    BOOL     fResult;

    hFile = CreateFile(pszKeyFile, fPreserveTimeStamp ? GENERIC_READ :
      GENERIC_WRITE, FILE_SHARE_READ | FILE_SHARE_WRITE, NULL,
      OPEN_EXISTING, 0, NULL);
    if (hFile == INVALID_HANDLE_VALUE) {
        return FALSE;
    }

    if (fPreserveTimeStamp) {
        if (!GetFileTime(hFile, &ftCreation, &ftLastAccess, &ftLastWrite)) {
            fPreserveTimeStamp = FALSE;
        }
    }

    hFile2 = CreateFile(pszKeyFile, GENERIC_READ, FILE_SHARE_READ | FILE_SHARE_WRITE,
      NULL, OPEN_EXISTING, 0, NULL);
    if (hFile2 != INVALID_HANDLE_VALUE) {
        fResult = TC_ReadKeyFile(paszPassword, hFile2);
        CloseHandle(hFile2);
    } else {
        fResult = FALSE;
    }

    if (fPreserveTimeStamp) {
        SetFileTime(hFile, &ftCreation, &ftLastAccess, &ftLastWrite);
    }

    CloseHandle(hFile);

    return fResult;
}

BOOL TC_ReadKeyFiles(PTC_PASSWORD pPw, LPCTSTR pszKeyFiles,
  BOOL fPreserveTimeStamps)
{
    TC_Assert(pPw);
    TC_Assert(pszKeyFiles);
    TC_Assert(*pszKeyFiles);

    for (; *pszKeyFiles; pszKeyFiles += _tcslen(pszKeyFiles) + 1) {
        DWORD dwAttr = GetFileAttributes(pszKeyFiles);
        if (dwAttr == 0xffffffff) {
            continue;
        }
        if (dwAttr & FILE_ATTRIBUTE_DIRECTORY) {
            TCHAR  szFileName[MAX_PATH];
            LPTSTR pszFilePart;
            WIN32_FIND_DATA fd;
            HANDLE hFindFile;

            if (!TC_CopyString(szFileName, MAX_PATH, pszKeyFiles)) {
                continue;
            }
            pszFilePart = szFileName + _tcslen(szFileName) - 1;
            if (*pszFilePart != TEXT('\\') && *pszFilePart != TEXT('/')) {
                if (!TC_AppendString(szFileName, MAX_PATH, TEXT("\\"))) {
                    continue;
                }
                pszFilePart++;
            }
            pszFilePart++;
            if (!TC_AppendString(szFileName, MAX_PATH, TEXT("*.*"))) {
                continue;
            }
            hFindFile = FindFirstFile(szFileName, &fd);
            if (hFindFile != INVALID_HANDLE_VALUE) {
                do {
                    if (fd.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY) {
                        continue;
                    }
                    if (!TC_CopyString(pszFilePart, MAX_PATH -
                      (DWORD)(pszFilePart - szFileName), fd.cFileName)) {
                        continue;
                    }
                    if (TC_ReadKeyFileByName(pPw->aszPassword, szFileName,
                      fPreserveTimeStamps)) {
                        pPw->nPasswordLen = TC_MAX_PASSWORD_LEN;
                    }
                } while (FindNextFile(hFindFile, &fd));
                FindClose(hFindFile);
            }
        } else {
            if (!TC_ReadKeyFileByName(pPw->aszPassword, pszKeyFiles,
              fPreserveTimeStamps)) {
                return FALSE;
            }
            pPw->nPasswordLen = TC_MAX_PASSWORD_LEN;
        }
    }

    return pPw->nPasswordLen != 0;
}