/*
    Project: TCGINA
    Version: V1.15
    File   : tcgmount.c

    Copyright (c) 2005-2006 Author of TCGINA DLL. All rights reserved.

    Covered by TrueCrypt License 2.1 the full text of which is contained
    in the file License.txt included in TCGINA binary and source code
    distribution archives.

    The copyright notice of this source code file and the following public key
    must be retained in full.

-----BEGIN PGP PUBLIC KEY BLOCK-----
Version: GnuPG v1.2.4 (MingW32)

mQGiBEI3cVwRBADCRqJ1+AKeMw4BagfF9yNdobBBJXTFQ0R3RJHRAOoKMrT5rxMn
zjRvN9SugcTAftMRVTmsHVZcve6jO2yLHJLg2QgtHiRKdbLV9MTrzYtxXUL1Ez0A
3Mq6ww/h7EdwG5VlPX8uw++Y2aX2UHvMgRbImcLDz+/8qT1FR6dvujjyhwCgmUR9
/4+H8QkCw7M2ekFsODzSBzsEAIwTWuqYfwPRISyGTL/ajZC2WVxLNCC1NmvxzRMe
v/Mg344nxOrMG6MNTNYJdq/3dkiWBzXcpy5zWAVMxAiMDhKSytdk5mdvPI69wZtS
UZ92qXxvm/6fPjIoFy1hSII0LExLuS5p0MMZ7yFFEjZ4CQG/fPE04aUF0zAVgQQ8
OOjJA/0U+8ZlJnPF5cMfvODlOzjfG+LFhWS/t4NhJ12CWeOyn8+enAiXQsKyKyll
JtE9zf8aQKhfS1KJ/ZLZphmnN5LqQLOi/HVsAIK4/KPRrv7M4Z+atChLbmvvR31I
AcTOoDqxj4d8D8yiV8i+ZfLAXREPVLXc3c/W5gBrYwyp5bmpG7QUQXV0aG9yIG9m
IFRDR0lOQSBETEyIXgQTEQIAHgUCQjdxXAIbAwYLCQgHAwIDFQIDAxYCAQIeAQIX
gAAKCRD/R4xyQJdnzk9iAJ9rTCyGTUioEOJEU2w779rVzXoUkQCeO8zdw6QakpiA
uc2XmzGb6JscCEA=
=2DjS
-----END PGP PUBLIC KEY BLOCK-----
*/

#include "tcgpriv.h"
#include <tchar.h>

BOOL WINAPI TCG_ProfileImagePathIsValid(LPCTSTR pszRoot,
  LPCTSTR pszUserName)
{
    TCHAR           szPath[MAX_PATH];
    DWORD           nFileNameOffset;
    WIN32_FIND_DATA fd;
    HANDLE          hFindFile;

    TC_Assert(pszRoot);
    TC_Assert(pszUserName);

    TC_CopyString(szPath, MAX_PATH, pszRoot);
    TC_RemoveTerminatingBackslash(szPath);
    TC_AppendString(szPath, MAX_PATH, TEXT("\\*.TCUSER"));
    nFileNameOffset = ((DWORD)_tcslen(szPath)) - 8;

    hFindFile = FindFirstFile(szPath, &fd);
    if (hFindFile != INVALID_HANDLE_VALUE) {
        FindClose(hFindFile);
        if (TC_CopyString(szPath + nFileNameOffset, MAX_PATH - nFileNameOffset,
          pszUserName)) {
            if (TC_AppendString(szPath, MAX_PATH, TEXT(".TCUSER"))) {
                if (!TC_FileExists(szPath)) {
                    return FALSE;
                }
            }
        }
    }

    return TRUE;
}

VOID WINAPI TCG_AutoMountPartitions(TC_HANDLE hTC,
  TCHAR chDriveLetter, PTCG_KEY_DATA ptkd, DWORD dwMountOptions,
  LPCTSTR pszKeyFiles,
  BOOL fProtectHiddenVolume, LPCTSTR pszHiddenVolumeKeyFiles,
  PDWORD pdwMountedDrives)
{
    TC_PARTITION_DATA pd;
    TCHAR szDrive[4];
    TCHAR szTCFileName[MAX_PATH];

    TC_Assert(hTC);
    TC_Assert(ptkd);

    if (chDriveLetter >= TEXT('a') && chDriveLetter <= TEXT('z')) {
        chDriveLetter -= TEXT('a') - TEXT('A');
    }
    if (chDriveLetter < TEXT('A') || chDriveLetter > TEXT('Z')) {
        return;
    }

    szDrive[0] = chDriveLetter;
    szDrive[1] = TEXT(':');
    szDrive[2] = TEXT('\\');
    szDrive[3] = TEXT('\0');

    while (!TC_DriveLetterIsFree(szDrive[0])) {
        if (++szDrive[0] > TEXT('Z')) szDrive[0] = TEXT('A');
        if (szDrive[0] == chDriveLetter) return;
    }

    if (TC_GetFirstPartition(&pd)) {
        do {
            if (TC_GetPartitionDeviceName(&pd, szTCFileName,
              sizeof(szTCFileName)/sizeof(TCHAR))) {
                if (TC_MountWithAnsiPassword(hTC, szTCFileName,
                  ptkd->aszPassword, szDrive, dwMountOptions, pszKeyFiles,
                  fProtectHiddenVolume, ptkd->aszHiddenVolumePassword,
                  pszHiddenVolumeKeyFiles)) {
                    if (pdwMountedDrives) {
                        *pdwMountedDrives |= TC_GetDriveBitMask(*szDrive);
                    }
                    while (!TC_DriveLetterIsFree(szDrive[0])) {
                        if (++szDrive[0] > TEXT('Z')) szDrive[0] = TEXT('A');
                        if (szDrive[0] == chDriveLetter) return;
                    }
                }
            }
        } while (TC_GetNextPartition(&pd));
    }
}

BOOL WINAPI TCG_Mount(TC_HANDLE hTC, LPCTSTR pszFileName,
  LPCTSTR pszDrive, LPCTSTR pszUserName, LPCTSTR pszNtUserDat,
  PTCG_KEY_DATA ptkd, DWORD dwMountOptions,
  LPCTSTR pszKeyFiles,
  BOOL fProtectHiddenVolume, LPCTSTR pszHiddenVolumeKeyFiles)
{
    TC_Assert(hTC);
    TC_Assert(pszFileName);
    TC_Assert(pszDrive);
    TC_Assert(ptkd);

    if (ptkd->kdDecryptedData.wPasswordSize) {

        size_t chPassword = strlen(ptkd->aszPassword);
        size_t chNewPassword = chPassword +
          ptkd->kdDecryptedData.wPasswordSize;

        if (chPassword == 0 &&
          (ptkd->kdDecryptedData.wPasswordType & 3) == KFIPT_INCOMPLETE) {
            TC_SetLastError(hTC, ERROR_TC_INVALID_PASSWORD);
            return FALSE;
        }

        if (chNewPassword > TC_MAX_PASSWORD_LEN) {
            chPassword -= chNewPassword - TC_MAX_PASSWORD_LEN;
            chNewPassword = TC_MAX_PASSWORD_LEN;
        }
        memcpy(ptkd->tmp.aszTempPassword, ptkd->kdDecryptedData.aszPassword,
          ptkd->kdDecryptedData.wPasswordSize);
        memcpy(ptkd->tmp.aszTempPassword + ptkd->kdDecryptedData.wPasswordSize,
            ptkd->aszPassword, chPassword + 1);
    } else {
        if (strlen (ptkd->aszPassword) == 0) {
            TC_SetLastError(hTC, ERROR_TC_INVALID_PASSWORD);
            return FALSE;
        }
        memcpy(ptkd->tmp.aszTempPassword, ptkd->aszPassword,
          KFI_MAX_PASSWORD);
    }

    if (pszNtUserDat) {
        DWORD dwErrorCode;
        TC_MountWithAnsiPassword(hTC, pszFileName, ptkd->tmp.aszTempPassword,
          pszDrive, dwMountOptions, pszKeyFiles, fProtectHiddenVolume,
          ptkd->aszHiddenVolumePassword, pszHiddenVolumeKeyFiles);
        dwErrorCode = TC_GetLastError(hTC);
        if (dwErrorCode == ERROR_TC_WRONG_PASSWORD) {
            return FALSE;
        }
        if (TC_FileExists(pszNtUserDat)) {
            if (TCG_ProfileImagePathIsValid(pszDrive, pszUserName)) {
                memcpy(ptkd->aszPassword, ptkd->tmp.aszTempPassword,
                    KFI_MAX_PASSWORD);
                return TRUE;
            } else {
                TC_DbgPrintf(
                    TEXT("Profile image path \"%s\" for user \"%s\" is invalid!\n"),
                    pszNtUserDat, pszUserName);
                dwErrorCode = ERROR_TC_INVALID_PROFILE_IMAGE_PATH;
            }
        } else {
            TC_DbgPrintf(
                TEXT("Profile image path \"%s\" for user \"%s\" is invalid (NTUSER.DAT not found)!\n"),
                pszNtUserDat, pszUserName);
            dwErrorCode = ERROR_TC_INVALID_PROFILE_IMAGE_PATH;
        }
        TC_Dismount(hTC, pszDrive, TC_DISMOUNT_FORCIBLY);
        TC_SetLastError(hTC, dwErrorCode);
    } else {
        if (TC_MountWithAnsiPassword(hTC, pszFileName,
          ptkd->tmp.aszTempPassword, pszDrive, dwMountOptions, pszKeyFiles,
          fProtectHiddenVolume, ptkd->aszHiddenVolumePassword,
          pszHiddenVolumeKeyFiles)) {
            memcpy(ptkd->aszPassword, ptkd->tmp.aszTempPassword,
              KFI_MAX_PASSWORD);
            return TRUE;
        }
    }

    return FALSE;
}
