/*
    Project: TCGINA
    Version: V1.15
    File   : tcgsharedmem.c

    Copyright (c) 2005-2006 Author of TCGINA DLL. All rights reserved.

    Covered by TrueCrypt License 2.1 the full text of which is contained
    in the file License.txt included in TCGINA binary and source code
    distribution archives.

    The copyright notice of this source code file and the following public key
    must be retained in full.

-----BEGIN PGP PUBLIC KEY BLOCK-----
Version: GnuPG v1.2.4 (MingW32)

mQGiBEI3cVwRBADCRqJ1+AKeMw4BagfF9yNdobBBJXTFQ0R3RJHRAOoKMrT5rxMn
zjRvN9SugcTAftMRVTmsHVZcve6jO2yLHJLg2QgtHiRKdbLV9MTrzYtxXUL1Ez0A
3Mq6ww/h7EdwG5VlPX8uw++Y2aX2UHvMgRbImcLDz+/8qT1FR6dvujjyhwCgmUR9
/4+H8QkCw7M2ekFsODzSBzsEAIwTWuqYfwPRISyGTL/ajZC2WVxLNCC1NmvxzRMe
v/Mg344nxOrMG6MNTNYJdq/3dkiWBzXcpy5zWAVMxAiMDhKSytdk5mdvPI69wZtS
UZ92qXxvm/6fPjIoFy1hSII0LExLuS5p0MMZ7yFFEjZ4CQG/fPE04aUF0zAVgQQ8
OOjJA/0U+8ZlJnPF5cMfvODlOzjfG+LFhWS/t4NhJ12CWeOyn8+enAiXQsKyKyll
JtE9zf8aQKhfS1KJ/ZLZphmnN5LqQLOi/HVsAIK4/KPRrv7M4Z+atChLbmvvR31I
AcTOoDqxj4d8D8yiV8i+ZfLAXREPVLXc3c/W5gBrYwyp5bmpG7QUQXV0aG9yIG9m
IFRDR0lOQSBETEyIXgQTEQIAHgUCQjdxXAIbAwYLCQgHAwIDFQIDAxYCAQIeAQIX
gAAKCRD/R4xyQJdnzk9iAJ9rTCyGTUioEOJEU2w779rVzXoUkQCeO8zdw6QakpiA
uc2XmzGb6JscCEA=
=2DjS
-----END PGP PUBLIC KEY BLOCK-----
*/

#define _WIN32_WINNT 0x0500
#include <windows.h>
#include <sddl.h>
#include "tcgpriv.h"

#ifdef SUPPORT_SHARED_MEMORY

static PTCG_SHARED_MEMORY psm = NULL;
static HANDLE hMapFile;
static HANDLE hMutex;

#define SHARED_MEMORY_NAME TEXT("TCGINA.{D127B54A-F278-4111-894F-F04F9F49A675}")
#define MUTEX_TIMEOUT      5000
#define MAX_PROFILE_COUNT  100

PTCG_SHARED_MEMORY WINAPI TCG_InitSharedMemory(PDWORD pdwProfileCount)
{
    SECURITY_ATTRIBUTES  sa;
    PSECURITY_ATTRIBUTES psa;
    DWORD                dwSharedMemorySize;

    TC_Assert(pdwProfileCount);

    if (psm != NULL) return psm;

    if (*pdwProfileCount == 0) {
        return NULL;
    }
    if (*pdwProfileCount > MAX_PROFILE_COUNT) {
        *pdwProfileCount = MAX_PROFILE_COUNT;
    }
    dwSharedMemorySize = sizeof(TCG_SHARED_MEMORY) +
      (*pdwProfileCount - 1) * sizeof(TCG_ENCRYPTED_PROFILE_INFO);

    sa.nLength = sizeof(sa);
    sa.bInheritHandle = FALSE;
    if (ConvertStringSecurityDescriptorToSecurityDescriptor(
      TEXT("D:")
      TEXT("(A;NP;GA;;;SY)")
      TEXT(""),
      SDDL_REVISION_1, &sa.lpSecurityDescriptor, NULL)) {
        psa = &sa;
    } else {
        sa.lpSecurityDescriptor = NULL;
        psa = NULL;
    }

    hMutex = CreateMutex(psa, FALSE,
      TEXT("Global\\") SHARED_MEMORY_NAME TEXT(".Mutex"));
    if (hMutex == NULL) {
        TC_DbgPrintfA("Cannot create mutex (ErrorCode=%d)\n",
          (int) GetLastError());
        LocalFree(sa.lpSecurityDescriptor);
        return NULL;
    }
    if (WaitForSingleObject(hMutex, MUTEX_TIMEOUT) != WAIT_OBJECT_0) {
        TC_DbgPrintfA("Cannot get mutex ownership\n");
        LocalFree(sa.lpSecurityDescriptor);
        return NULL;
    }

    hMapFile = CreateFileMapping(INVALID_HANDLE_VALUE, psa, PAGE_READWRITE, 0,
      dwSharedMemorySize, TEXT("Global\\") SHARED_MEMORY_NAME);
    if (hMapFile == NULL) {
        TC_DbgPrintfA("Cannot create file mapping (ErrorCode=%d)\n",
            (int)GetLastError());
        ReleaseMutex(hMutex);
        LocalFree(sa.lpSecurityDescriptor);
        CloseHandle(hMutex);
        return NULL;
    }
    LocalFree(sa.lpSecurityDescriptor);

    psm = (PTCG_SHARED_MEMORY) MapViewOfFile(hMapFile, FILE_MAP_WRITE, 0, 0,
      dwSharedMemorySize);
    if (psm == NULL) {
        TC_DbgPrintfA("MapViewOfFile failed (ErrorCode=%d)\n",
          GetLastError());
        ReleaseMutex(hMutex);
        CloseHandle(hMapFile);
        CloseHandle(hMutex);
        return NULL;
    }
    if (psm->dwProfileCount == 0) {
        psm->dwProfileCount = *pdwProfileCount;
    } else {
        *pdwProfileCount = psm->dwProfileCount;
    }

    ReleaseMutex(hMutex);

    return psm;
}

BOOL WINAPI TCG_LockSharedMemory(void)
{
    if (psm == NULL) {
        return FALSE;
    }

    if (WaitForSingleObject(hMutex, MUTEX_TIMEOUT) != WAIT_OBJECT_0) {
        TC_DbgPrintfA("Cannot lock shared memory!\n");
        return FALSE;
    }

    return TRUE;
}

BOOL WINAPI TCG_UnlockSharedMemory(void)
{
    if (psm == NULL) {
        return FALSE;
    }

    return ReleaseMutex(hMutex);
}

#if 0
VOID WINAPI TCG_FreeSharedMemory(void)
{
    if (psm) {
        UnmapViewOfFile((LPCVOID)psm);
        CloseHandle(hMapFile);
        CloseHandle(hMutex);
        psm = NULL;
    }
}
#endif

#endif
