/*
 * File   : repsha1.c
 * Version: 1.0
 *
 * THIS SOFTWARE IS NOT COPYRIGHTED
 *
 * This source code is offered for use in the public domain. You may
 * use, modify or distribute it freely.
 *
 * This code is distributed in the hope that it will be useful but
 * WITHOUT ANY WARRANTY. ALL WARRANTIES, EXPRESS OR IMPLIED ARE HEREBY
 * DISCLAIMED. This includes but is not limited to warranties of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <io.h>
#include "tomcrypt.h"

#define ERROR_SYNTAX 1
#define ERROR_FILE   2

typedef char *PFILE_DATA;

BOOL LoadFile(const char *pszFileName, long *plFileSize, 
  PFILE_DATA *ppFileData)
{
	FILE *fp;

	fp = fopen(pszFileName, "rb");
	if (fp == NULL) {
		return FALSE;
	}

	*plFileSize = filelength(fileno(fp));
	if (*plFileSize <= 0) {
		fclose(fp);
		return FALSE;
	}

	/* allocate one more byte for NUL termination */
	*ppFileData = malloc(*plFileSize + 1);
	if (*ppFileData == NULL) {
		fclose(fp);
		return FALSE;
	}
	(*ppFileData)[*plFileSize] = '\0';

	if (fread(*ppFileData, *plFileSize, 1, fp) != 1) {
		free(*ppFileData);
		fclose(fp);
		return FALSE;
	}

	fclose(fp);

	return TRUE;
}

int main(int argc, char *argv[])
{
	PFILE_DATA pSrcFileData;
	long       lFileSize;
	FILE       *fp;
	PFILE_DATA p;
	PFILE_DATA pEnd;
	
	if (argc != 3) {
		puts("REPSHA1 V1.0\n"
		     "SYNTAX: REPSHA1 src dest\n"
        );
		return ERROR_SYNTAX;
	}

	if (!LoadFile(argv[1], &lFileSize, &pSrcFileData)) {
		puts("Cannot load input file!");
		return ERROR_FILE;
	}
	fp = fopen(argv[2], "wb");
	if (fp == NULL) {
		puts("Cannot create output file!");
		return ERROR_FILE;
	}

	p = pSrcFileData;
	pEnd = p + lFileSize;
	for (;;) {
		PFILE_DATA pFileData;
		char *q = strstr(p, "$SHA1(");
		if (q == NULL) break;
		fwrite(p, q - p, 1, fp);
		p = q + 6;
		q = strchr(p, ')');
		if (q == NULL) break;
		*q = '\0';
		if (LoadFile(p, &lFileSize, &pFileData)) {
			hash_state    md;
			unsigned char rgbHash[20];
			unsigned      i;
			sha1_init(&md);
			sha1_process(&md, pFileData, lFileSize);
			sha1_done(&md, rgbHash);
			for (i = 0; i < sizeof(rgbHash); i++) {
				fprintf(fp, "%02X", (unsigned) rgbHash[i]);
			}
			free(pFileData);
		}
		p = q + 1;
	}
	if (p != pEnd) {
		fwrite(p, pEnd - p, 1, fp);
	}

	/* resources are freed by the OS when the process is terminated */

	return 0;
}
