@echo off

rem ***************************************************************************
rem ** Environment settings                                                  **
rem ***************************************************************************

rem Microsoft Visual Studio .NET 2003 (version 7.1) or compatible
set DEVENV=%ProgramFiles%\Microsoft Visual Studio .NET 2003\Common7\IDE\devenv.exe

rem Platform SDK for Windows Server 2003 R2 March 2006 Edition or compatible
set SDK=%ProgramFiles%\Microsoft Platform SDK for Windows Server 2003 R2

rem Temporary drive letter
set DRIVE=S:

rem ***************************************************************************
rem ** Check environment settings                                            **
rem ***************************************************************************

set ARG0=%~0
if exist "%DEVENV%" goto :devenv_exists
cls
echo.Cannot find Microsoft Visual Studio .NET 2003
set INPUT=
set /P INPUT=Do you want to edit the environment settings (y/n)? 
if "%INPUT%" equ "y" goto :edit_settings
:devenv_exists

if exist "%SDK%\setenv.cmd" goto :sdk_exists
cls
echo.Cannot find Platform SDK for Windows Server 2003
set INPUT=
set /P INPUT=Do you want to edit the environment settings (y/n)? 
if "%INPUT%" equ "y" goto :edit_settings
:sdk_exists

rem ***************************************************************************
rem ** Project settings                                                      **
rem ***************************************************************************

set TCGINA_FILE_VERSION=115

rem ***************************************************************************
rem ** Remove output directories                                             **
rem ***************************************************************************

for /d %%i in (Release*) do rd /s /q "%%i"
for /d %%i in (Debug*) do rd /s /q "%%i"
for /d %%i in (bin*) do rd /s /q "%%i"

rem ***************************************************************************
rem ** Map current directory to temporary drive letter                       **
rem ***************************************************************************

set HOMEDIR=%CD%
subst %DRIVE% /d
subst %DRIVE% .
if errorlevel 1 goto :invalid_drive
cd /d %drive%\

rem ***************************************************************************
rem ** Compile project                                                       **
rem ***************************************************************************

cls
echo.Compiling TCGINA LITE...
call :build x86 release tcgina tcgina-lite
echo.Compiling TCGINA FULL...
call :build x86 release tcgina tcgina
echo.Compiling TCGINA DEBUG...
call :build x86 debug   tcgina tcgina
echo.Compiling SETUP...
call :build x86 release tcgina tcgina-setup
echo.Compiling KFEDIT...
call :build x86 release tcgina kfedit
echo.Compiling REG...
call :build x86 release reg reg\reg

echo.Compiling TCGINA LITE (x64)...
call :build x64 release tcgina tcgina-lite
echo.Compiling TCGINA FULL (x64)...
call :build x64 release tcgina tcgina
echo.Compiling TCGINA DEBUG (x64)...
call :build x64 debug   tcgina tcgina
echo.Compiling SETUP (x64)...
call :build x64 release tcgina tcgina-setup
echo.Compiling KFEDIT (x64)...
call :build x64 release tcgina kfedit
echo.Compiling REG (x64)...
call :build x64 release reg reg\reg

echo.Compiling REPSHA1...
call :make tools\repsha1\repsha1 repsha1

rem ***************************************************************************
rem ** Copy files to redistributable directories                             **
rem ***************************************************************************

set BIN=bin
set REL=Release
set DBG=Debug
set EXT=
call :copy_bin
tools\repsha1\release\repsha1 setup\setup.ini "%BIN%\install\setup.ini"

set BIN=bin-x64
set REL=Release x64
set DBG=Debug x64
set EXT=-x64
call :copy_bin
tools\repsha1\release\repsha1 setup\setup-x64.ini "%BIN%\install\setup.ini"

goto :exit

:copy_bin
md "%BIN%"
md "%BIN%\install"
md "%BIN%\kfedit"
md "%BIN%\tcuser"
copy "%REL%\setup.exe" "%BIN%\install\setup.exe"
copy "%REL%\tcgina-lite.dll" "%BIN%\install\tcgl%TCGINA_FILE_VERSION%%EXT%.bin"
copy "%REL%\tcgina.dll" "%BIN%\install\tcgf%TCGINA_FILE_VERSION%%EXT%.bin"
copy "%DBG%\tcgina.dll" "%BIN%\install\tcgd%TCGINA_FILE_VERSION%%EXT%.bin"
copy "%REL%\kfedit.exe" "%BIN%\kfedit\kfedit.exe"
copy "%REL%\reg.exe" "%BIN%\tcuser\reg.exe"

goto :eof

rem ***************************************************************************
rem ** Build x86|x64 release|debug solution project                          **
rem ***************************************************************************

:build
set SOLUTION=%~3
set PROJECT=%~4
if "%1" equ "x86" goto :x86
if "%1" equ "x64" goto :x64
goto :eof
:x86
if "%2" equ "release" goto :x86_release
if "%2" equ "debug" goto :x86_debug
goto :eof
:x86_release
"%DEVENV%" "%SOLUTION%.sln" /rebuild "Release" /project "%PROJECT%.vcproj"
goto :eof
:x86_debug
"%DEVENV%" "%SOLUTION%.sln" /rebuild "Debug" /project "%PROJECT%.vcproj"
goto :eof
:x64
if "%2" equ "release" goto :x64_release
if "%2" equ "debug" goto :x64_debug
goto :eof
:x64_release
call :setsdkenv /XP64 /RETAIL
"%DEVENV%" "%SOLUTION%.sln" /rebuild "Release x64" /project "%PROJECT%.vcproj" /useenv
goto :eof
:x64_debug
call :setsdkenv /XP64 /DEBUG
"%DEVENV%" "%SOLUTION%.sln" /rebuild "Debug x64" /project "%PROJECT%.vcproj" /useenv
goto :eof

rem ***************************************************************************
rem ** Make solution project                                                 **
rem ***************************************************************************

:make
set SOLUTION=%~1
set PROJECT=%~2
"%DEVENV%" "%SOLUTION%.sln" /build "Release" /project "%PROJECT%.vcproj"
goto :eof

rem ***************************************************************************
rem ** Set SDK environment                                                   **
rem ***************************************************************************

:setsdkenv
SET _HOMEDIR=%CD%
cd /d "%SDK%"
call setenv %1 %2 %3 %4 %5 %6 %7 %8 %9 >nul
cd /d "%_HOMEDIR%"
set _HOMEDIR=
goto :eof

rem ***************************************************************************
rem ** Edit environment settings                                             **
rem ***************************************************************************
:edit_settings
notepad "%ARG0%"
goto :eof

rem ***************************************************************************
rem ** Error messages                                                        **
rem ***************************************************************************

:invalid_drive
cls
echo.ERROR: Drive letter '%DRIVE%' is not free!
pause
goto :exit

rem ***************************************************************************
rem ** Exit                                                                  **
rem ***************************************************************************
:exit
subst %DRIVE% /d
cd /d "%HOMEDIR%"
